@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveTraversable #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE FlexibleInstances #-}
@@ -44,9 +45,10 @@ import Control.Exception (fromException, try, bracket_, throw, finally, SomeExce
 import qualified Control.Exception as E
 import Control.Applicative ((<|>), empty)
 import Control.Monad.Fail
+import Control.Monad.Fix (fix)
 import Control.Monad.State
 import Control.Monad.Reader
-import Control.Monad (void)
+import Control.Monad (filterM, guard, liftM2, void, when)
 import Data.Semigroup
 import Data.Traversable (for)
 import Data.Time.Clock (UTCTime)
@@ -451,7 +453,11 @@ xfork x = io . forkProcess . finally nullStdin $ do
                 x
  where
     nullStdin = do
+#if MIN_VERSION_unix(2,8,0)
+        fd <- openFd "/dev/null" ReadOnly defaultFileFlags
+#else
         fd <- openFd "/dev/null" ReadOnly Nothing defaultFileFlags
+#endif
         dupTo fd stdInput
         closeFd fd
 
